
sampler2D noiseSampler;
sampler2D fireSampler;
sampler2D fogOfWarSampler;

void vs_main(
  float4 inPosition : POSITION0,
  float3 inNormal : NORMAL0,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION0,
  out float2 outTexCoord : TEXCOORD0,
  out float2 outTexCoordNoise1 : TEXCOORD1,
  out float2 outTexCoordNoise2 : TEXCOORD2,
  out float2 outTexCoordNoise3 : TEXCOORD3,
  out float outMainAlpha : TEXCOORD4,
  out float2 outFogOfWarTexCoord : TEXCOORD5,
  uniform float4x4 matWorld,
  uniform float4x4 matViewProjection,
  uniform float3 dirToCam,
  uniform float time,
  uniform float offset,
  uniform float2 mapSize
)
{
  // Project position
  float4 worldPosition = mul(matWorld, inPosition);
  outPosition = mul(matViewProjection, worldPosition);

  //copy main UV coordinates across 
  outTexCoord = inTexCoord;

  //Generate noise coordinates as a function of time
  outTexCoordNoise1 = inTexCoord + float2(time * 0.7f + offset, time);
  outTexCoordNoise2 = inTexCoord + float2(-time * 0.11f, time * 0.93f + offset);
  outTexCoordNoise3 = inTexCoord + float2(time * 0.3f - offset, time * 0.71f);

  //Transform normal
  inNormal = mul(matWorld, inNormal);

  //Calculate mainAlpha
  outMainAlpha = abs(dot(normalize(inNormal), dirToCam));

  // FogOfWar
  outFogOfWarTexCoord = float2(worldPosition.x / mapSize.x, -worldPosition.z / mapSize.y);
}

void ps_main(
  float2 inTexCoord : TEXCOORD0,
  float2 inTexCoordNoise1 : TEXCOORD1,
  float2 inTexCoordNoise2 : TEXCOORD2,
  float2 inTexCoordNoise3 : TEXCOORD3,
  float inMainAlpha : TEXCOORD4,
  float2 inFogOfWarTexCoord : TEXCOORD5,
  out float4 outColor : COLOR
)
{
  //Sample noise pixels
  float n1 = tex2D(noiseSampler, inTexCoordNoise1);
  float n2 = tex2D(noiseSampler, inTexCoordNoise2);
  float n3 = tex2D(noiseSampler, inTexCoordNoise3);

  //Sum up noise and perturb the x coordinate of the main UV
  inTexCoord.x += (n1 + n2 + n3) * 0.1f - 0.05f;

  //Sample the diffuse texture
  float4 c0 = tex2D(fireSampler, inTexCoord);

  //Multiply mainAlpha with this pixels alpha value
  c0.a *= inMainAlpha;

  outColor = c0;

  // apply fog of war
  float4 fogOfWar = tex2D(fogOfWarSampler, inFogOfWarTexCoord);
  outColor *= fogOfWar;
}