
sampler2D particleSampler;
sampler2D fogOfWarSampler;

float compute_distance_to_viewer(
  float3 worldPosition,
  float3 worldEye
)
{
  float3 worldEyeToWorldPosition = worldPosition - worldEye;
  return sqrt(dot(worldEyeToWorldPosition, worldEyeToWorldPosition));
}

float compute_point_size(
  float Vh, // viewport height
  float Si, // pointsize input
  float A, // pointscale parameter 1
  float B, // pointscale parameter 2
  float C, // pointscale parameter 3
  float De // distance between viewer and point
)
{
  return Vh * Si * sqrt(1 / (A + B*De + C*(De*De)));
}

/*
	Speziell ist die Punktgrssenberechung. Sie wird mit der folgenden Formel berechnet:
	S = Vh * Si * sqrt(1/(A + B * De + C *( De^2 )))
	
	Dabei werden die folgenden Parameter verwendet:
		Vh: Viewport  Height
		Si: Input Pointgrsse
		A, B, C: Parameter
		De: Distanz zwischen Punkt und Betrachter

	
*/
void vs_main(
  float3 worldPosition : POSITION,
  float inPointSize : PSIZE,
  float4 inColor : COLOR0,
  out float4 outPosition : POSITION,
  out float4 outColor : COLOR0,
  out float outPointSize : PSIZE,
  out float2 outFogOfWarTexCoord : TEXCOORD1,
  uniform float4 pointScaleParameters,
  uniform float3 worldEye,
  uniform float4x4 matViewProjection,
  uniform float2 mapSize
)
{
  // distanz zwischen betrachter und punkt berechnen
  float De = compute_distance_to_viewer(worldPosition, worldEye);

  // punktgrsse bestimmen
  float Vh = pointScaleParameters.x;
  float Si = inPointSize;
  float A = pointScaleParameters.y;
  float B = pointScaleParameters.z;
  float C = pointScaleParameters.w;
  float S = compute_point_size(Vh, Si, A, B, C, De);

  outPosition = mul(matViewProjection, float4(worldPosition, 1));
  outColor = inColor;
  outPointSize = S;
  outFogOfWarTexCoord = float2(worldPosition.x / mapSize.x, -worldPosition.z / mapSize.y);
}

void ps_main(
  float4 inColor : COLOR0,
  float2 inTexCoord : TEXCOORD0,
  float2 inFogOfWarTexCoord : TEXCOORD1,
  out float4 outColor : COLOR
)
{
  outColor = inColor * tex2D(particleSampler, inTexCoord);
  float4 fogOfWar = tex2D(fogOfWarSampler, inFogOfWarTexCoord);
  outColor *= fogOfWar;
}
