//--------------------------------------------------------------------------------------------------
// samplers used in all three shaders
sampler sightSampler;
sampler visibleSampler;
sampler visitedSampler;




//--------------------------------------------------------------------------------------------------
// shader to render the visible texture
void visible_vs_main(
  float3 inPosition : POSITION,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION,
  out float2 outTexCoord : TEXCOORD0,
  uniform float4x4 matWorldViewProjection
)
{
  outPosition = mul(matWorldViewProjection, float4(inPosition, 1));
  outTexCoord = inTexCoord;
}

void visible_ps_main(
  float2 inTexCoord : TEXCOORD0,
  out float4 outColor : COLOR
)
{
  outColor = tex2D(sightSampler, inTexCoord);
}




//--------------------------------------------------------------------------------------------------
// shader to render the visited texture
void visited_vs_main(
  float4 inPosition : POSITION,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION,
  out float2 outTexCoord : TEXCOORD0,
  uniform float4x4 orthogonalProjection
)
{
  outPosition = mul(orthogonalProjection, inPosition);
  outTexCoord = inTexCoord;
}

void visited_ps_main(
  float2 inTexCoord : TEXCOORD0,
  out float4 outColor : COLOR
)
{
  float4 visibleColor = tex2D(visibleSampler, inTexCoord);
  float4 visitedColor = tex2D(visitedSampler, inTexCoord);
  outColor = max(visibleColor, visitedColor);
}




//--------------------------------------------------------------------------------------------------
// shader to render the fog-of-war texture
void fog_of_war_vs_main(
  float4 inPosition : POSITION,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION,
  out float2 outTexCoord : TEXCOORD0,
  uniform float4x4 orthogonalProjection
)
{
  outPosition = mul(orthogonalProjection, inPosition);
  outTexCoord = inTexCoord;
}

void fog_of_war_ps_main(
  float2 inTexCoord : TEXCOORD0,
  out float4 outColor : COLOR
)
{
  float4 visibleColor = tex2D(visibleSampler, inTexCoord);
  float4 visitedColor = tex2D(visitedSampler, inTexCoord);
  outColor = max(visibleColor, visitedColor * 0.4);
}
