//--------------------------------------------------------------------------------------------------
// shaders to render the minimap                                                                  //
// author: Christian Oberholzer                                                                   //
//--------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------
sampler2D landscapeSampler;
sampler2D unitLayerSampler;
sampler2D fogOfWarSampler;




//--------------------------------------------------------------------------------------------------
void unit_layer_vs_main(
  float3 inPosition : POSITION,
  out float4 outPosition : POSITION,
  uniform float4x4 matWorldViewProjection
)
{
  outPosition = mul(matWorldViewProjection, float4(inPosition, 1));
}

void unit_layer_ps_main(
  out float4 outColor : COLOR,
  uniform float4 teamColor
)
{
  outColor = teamColor;
}




//--------------------------------------------------------------------------------------------------
void minimap_vs_main(
  float4 inPosition : POSITION,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION,
  out float2 outTexCoord : TEXCOORD0,
  uniform float4x4 orthogonalProjection
)
{
  outPosition = mul(orthogonalProjection, inPosition);
  outTexCoord = inTexCoord;
}

void minimap_ps_main(
  float2 inTexCoord : TEXCOORD0,
  out float4 outColor : COLOR
)
{
  float4 landscape = tex2D(landscapeSampler, inTexCoord);
  float4 unitLayer = tex2D(unitLayerSampler, inTexCoord);
  float4 fogOfWar = tex2D(fogOfWarSampler, inTexCoord);
  
  outColor = any(unitLayer.rgb) ? unitLayer : landscape;
  outColor *= fogOfWar;
}
