sampler diffuseSampler;
sampler fogOfWarSampler;

struct DirectionalLight
{
  float3 direction;
  float4 color;
};

struct Material
{
  float4 ambientColor;
  float4 diffuseColor;
  //float4 specularColor;
  //float4 emissiveColor;
};

void vs_main(
  float3 inObjectPosition : POSITION,
  float3 inNormal : NORMAL,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION,
  out float3 outNormal : TEXCOORD2,
  out float2 outTexCoord : TEXCOORD0,
  out float2 outFogOfWarTexCoord : TEXCOORD1,
  uniform float4x4 matWorld,
  uniform float4x4 matView,
  uniform float4x4 matProjection,
  uniform float2 mapSize
)
{
  float4 worldPosition = mul(matWorld, float4(inObjectPosition, 1));
  float4 viewPosition = mul(matView, worldPosition);
  float4 screenPosition = mul(matProjection, viewPosition);
  
  outPosition = screenPosition;
  outTexCoord = inTexCoord;
  outNormal = mul(matWorld, inNormal);
  outFogOfWarTexCoord = float2(worldPosition.x / mapSize.x, -worldPosition.z / mapSize.y);
}

void ps_main(
  float3 inNormal : TEXCOORD2,
  float2 inTexCoord : TEXCOORD0,
  float2 inFogOfWarTexCoord : TEXCOORD1,
  out float4 outColor : COLOR,
  uniform Material material,
  uniform float4 ambientLight,
  uniform DirectionalLight light
)
{
  // fetch object textures
  float4 texColor = tex2D(diffuseSampler, inTexCoord);
  float4 fogOfWar = tex2D(fogOfWarSampler, inFogOfWarTexCoord);

  // calculate unlighted colors
  float4 ambientColor = material.ambientColor * texColor;
  float4 diffuseColor = material.diffuseColor * texColor;

  // calculate lighting 
  inNormal = normalize(inNormal);
  float3 toLight = normalize(-light.direction);
  float lambertTerm = saturate(dot(inNormal, toLight));
  float4 diffuseLight = light.color * lambertTerm;

  // combine light-colors and unlighted colors  
  float4 lightedValue = (
    ambientColor * ambientLight +
    diffuseColor * diffuseLight
  );
  
  // now combine the lighted value with the fog-of-war
  outColor = lightedValue * fogOfWar;
}
