sampler diffuseSampler;

struct DirectionalLight
{
  float3 direction;
  float4 color;
};

struct Material
{
  float4 ambientColor;
  float4 diffuseColor;
  //float4 specularColor;
  //float4 emissiveColor;
};

void vs_main(
  float3 inPosition : POSITION,
  float3 inNormal : NORMAL,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION,
  out float3 outNormal : TEXCOORD1,
  out float2 outTexCoord : TEXCOORD0,
  uniform float4x4 matWorld,
  uniform float4x4 matView,
  uniform float4x4 matProjection
)
{
  outPosition = mul(matWorld, float4(inPosition, 1));
  outPosition = mul(matView, outPosition);
  outPosition = mul(matProjection, outPosition);
  outTexCoord = inTexCoord;
  outNormal = mul(matWorld, inNormal);
}

void ps_main(
  float3 inNormal : TEXCOORD1,
  float2 inTexCoord : TEXCOORD0,
  out float4 outColor : COLOR,
  uniform Material material,
  uniform float4 ambientLight,
  uniform DirectionalLight light
)
{
  inNormal = normalize(inNormal);
  float3 toLight = normalize(-light.direction);
  float lambertTerm = saturate(dot(inNormal, toLight));
  float4 diffuseLight = light.color * lambertTerm;

  float4 lighted = (
    material.ambientColor * ambientLight +
    material.diffuseColor * diffuseLight
  );

  float4 texColor = tex2D(diffuseSampler, inTexCoord);

  outColor = lighted * texColor;
}
