sampler diffuseSampler;

struct DirectionalLight
{
  float3 direction;
  float4 color;
};

struct Material
{
  float4 ambientColor;
  float4 diffuseColor;
  //float4 specularColor;
  //float4 emissiveColor;
};

void vs_main(
  float3 inPosition : POSITION,
  float3 inNormal : NORMAL,
  float2 inTexCoord : TEXCOORD0,
  out float4 outPosition : POSITION,
  out float3 outNormal : TEXCOORD1,
  out float2 outTexCoord : TEXCOORD0,
  uniform float4x4 matWorld,
  uniform float4x4 matInvTransposeWorld,
  uniform float4x4 matView,
  uniform float4x4 matProjection
)
{
  outPosition = mul(matWorld, float4(inPosition, 1));
  outPosition = mul(matView, outPosition);
  outPosition = mul(matProjection, outPosition);
  outTexCoord = inTexCoord;
  outNormal = mul(matInvTransposeWorld, inNormal);
}

void ps_main(
  float3 inNormal : TEXCOORD1,
  float2 inTexCoord : TEXCOORD0,
  out float4 outColor : COLOR,
  uniform Material material,
  uniform float4 ambientLight,
  uniform DirectionalLight light,
  uniform float4 teamColor
)
{
  // fetch building texture
  float4 texColor = tex2D(diffuseSampler, inTexCoord);
  
  // calculate weighting of tex/material-color and team-color
  float teamColorWeight = texColor.a;
  float materialWeight = 1.0 - teamColorWeight;

  // calculate unlighted colors
  float4 ambientColor = float4((material.ambientColor * texColor).rgb * materialWeight + teamColor.rgb * teamColorWeight, 1.0);
  float4 diffuseColor = float4((material.diffuseColor * texColor).rgb * materialWeight + teamColor.rgb * teamColorWeight, 1.0);

  // calculate lighting 
  inNormal = normalize(inNormal);
  float3 toLight = normalize(-light.direction);
  float lambertTerm = saturate(dot(inNormal, toLight));
  float4 diffuseLight = light.color * lambertTerm;

  // combine light-colors and unlighted colors  
  outColor = (
    ambientColor * ambientLight +
    diffuseColor * diffuseLight
  );
}
